@@ -41,8 +41,7 @@
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtksignal.h>
+#include <gtk/gtk.h>
 
 enum {
 	FILES_ADDED,
@@ -572,7 +571,9 @@ nautilus_directory_is_local (NautilusDirectory *directory)
 	if (directory->details->location == NULL) {
 		return TRUE;
 	}
-	return g_file_is_native (directory->details->location);
+
+	return nautilus_directory_is_in_trash (directory) ||
+	       g_file_is_native (directory->details->location);
 }
 
 gboolean
@@ -632,6 +633,7 @@ void
 nautilus_directory_add_file (NautilusDirectory *directory, NautilusFile *file)
 {
 	GList *node;
+	gboolean add_to_work_queue;
 
 	g_assert (NAUTILUS_IS_DIRECTORY (directory));
 	g_assert (NAUTILUS_IS_FILE (file));
@@ -646,9 +648,19 @@ nautilus_directory_add_file (NautilusDirectory *directory, NautilusFile *file)
 
 	directory->details->confirmed_file_count++;
 
-	/* Ref if we are monitoring. */
+	add_to_work_queue = FALSE;
 	if (nautilus_directory_is_file_list_monitored (directory)) {
+		/* Ref if we are monitoring, since monitoring owns the file list. */
 		nautilus_file_ref (file);
+		add_to_work_queue = TRUE;
+	} else if (nautilus_directory_has_active_request_for_file (directory, file)) {
+		/* We're waiting for the file in a call_when_ready. Make sure
+		   we add the file to the work queue so that said waiter won't
+		   wait forever for e.g. all files in the directory to be done */
+		add_to_work_queue = TRUE;
+	}
+	
+	if (add_to_work_queue) {
 		nautilus_directory_add_file_to_work_queue (directory, file);
 	}
 }
@@ -1592,6 +1604,9 @@ nautilus_directory_schedule_position_set (GList *position_setting_list)
 	const NautilusFileChangesQueuePosition *item;
 	NautilusFile *file;
 	char str[64];
+	time_t now;
+
+	time (&now);
 
 	for (p = position_setting_list; p != NULL; p = p->next) {
 		item = (NautilusFileChangesQueuePosition *) p->data;
@@ -1609,6 +1624,18 @@ nautilus_directory_schedule_position_set (GList *position_setting_list)
 			 NULL,
 			 str);
 
+		if (item->set) {
+			nautilus_file_set_time_metadata
+				(file,
+				 NAUTILUS_METADATA_KEY_ICON_POSITION_TIMESTAMP,
+				 now);
+		} else {
+			nautilus_file_set_time_metadata
+				(file,
+				 NAUTILUS_METADATA_KEY_ICON_POSITION_TIMESTAMP,
+				 UNDEFINED_TIME);
+		}
+
 		if (item->set) {
 			g_snprintf (str, sizeof (str), "%d", item->screen);
 		} else {
